import axios from 'axios'

// 创建axios实例
const apiClient = axios.create({
  baseURL: '', // 使用相对路径，将通过Vite代理
  timeout: 15000, // 请求超时时间
  headers: {
    'Content-Type': 'application/json',
    'Accept': 'application/json'
  }
})

// 添加请求拦截器
apiClient.interceptors.request.use(
  config => {
    console.log('API请求:', {
      url: config.url,
      method: config.method,
      params: config.params,
      data: config.data,
      baseURL: config.baseURL
    });
    return config;
  },
  error => {
    console.error('请求拦截器错误:', error);
    return Promise.reject(error);
  }
)

// 添加响应拦截器
apiClient.interceptors.response.use(
  response => {
    console.log('API响应成功:', {
      url: response.config.url,
      status: response.status,
      data: response.data
    });
    return response;
  },
  error => {
    console.error('API响应错误:', {
      url: error.config?.url,
      message: error.message,
      response: error.response?.data
    });
    return Promise.reject(error);
  }
)

// 定义接口
interface PageItem {
  id?: number;
  formatId?: string;
  title?: string;
  content?: string;
  pageType?: string;
  pageUrl?: string;
  createTime?: string;
  updateTime?: string;
  status?: number;
  sortOrder?: number;
  viewCount?: number;
  author?: string;
  keywords?: string;
  description?: string;
  multiAttachments?: string;
  attachmentUrl?: string;
  [key: string]: any; // 允许其他属性
}

// 获取页面详情
export const getPageDetail = async (type: string, formatId: string) => {
  try {
    console.log(`开始请求${type}详情，formatId:`, formatId);
    
    // 使用带有pageType和formatId参数的路径，确保获取正确类型的内容
    const url = `/hasfj/hasfjpages/list`;
    const params = { 
      pageType: type,
      formatId: formatId,
      pageNum: 1,
      pageSize: 10
    };
    console.log(`请求详情URL:`, url, '参数:', params);
    
    const response = await apiClient.get(url, { params });
    console.log(`获取详情响应:`, response);
    
    if (response.status === 200 && response.data) {
      console.log(`获取${type}详情成功:`, response.data);
      
      // 检查响应数据结构
      let resultData = null;
      
      // 处理数组格式的响应
      if (Array.isArray(response.data)) {
        // 从数组中找到匹配的记录
        const matchedItem = response.data.find((item: PageItem) => 
          item.formatId === formatId && (!item.pageType || item.pageType === type)
        );
        
        if (matchedItem) {
          console.log(`找到匹配的${type}数据:`, matchedItem);
          resultData = matchedItem;
        }
      } 
      // 处理标准响应格式
      else if (response.data.rows && Array.isArray(response.data.rows)) {
        const matchedItem = response.data.rows.find((item: PageItem) => 
          item.formatId === formatId && (!item.pageType || item.pageType === type)
        );
        
        if (matchedItem) {
          console.log(`找到匹配的${type}数据:`, matchedItem);
          resultData = matchedItem;
        }
      }
      // 处理单个对象响应
      else if (response.data.formatId === formatId) {
        resultData = response.data;
      }
      
      if (resultData) {
        // 验证返回的数据类型是否正确
        if (resultData.pageType && resultData.pageType !== type) {
          console.warn(`API返回了错误的内容类型: 请求${type}但返回${resultData.pageType}`);
          // 尝试使用备用方法获取正确类型的数据
          return await requestWithCorrectType(type, formatId);
        }
        
        return {
          code: 200,
          msg: '获取成功',
          data: resultData
        };
      } else {
        console.error(`未找到匹配的${type}数据，formatId:`, formatId);
        // 尝试使用备用方法获取正确类型的数据
        return await requestWithCorrectType(type, formatId);
      }
    } else {
      console.error(`获取${type}详情失败:`, response);
      throw new Error(`获取详情失败，状态码: ${response.status}`);
    }
  } catch (error) {
    console.error(`获取${type}详情失败:`, error);
    
    // 尝试使用备用方法获取正确类型的数据
    return await requestWithCorrectType(type, formatId);
  }
}

// 使用正确的类型参数请求数据
async function requestWithCorrectType(type: string, formatId: string) {
  try {
    // 尝试使用明确指定类型的API路径
    console.log(`尝试使用明确指定类型的API路径`);
    const url = `/hasfj/hasfjpages/list`;
    const params = { 
      pageType: type,
      pageNum: 1,
      pageSize: 100
    };
    
    const response = await apiClient.get(url, { params });
    
    if (response.status === 200 && response.data) {
      let items: PageItem[] = [];
      
      // 提取数据项
      if (Array.isArray(response.data)) {
        items = response.data;
      } else if (response.data.rows && Array.isArray(response.data.rows)) {
        items = response.data.rows;
      } else if (response.data.data && Array.isArray(response.data.data)) {
        items = response.data.data;
      }
      
      // 过滤掉不匹配类型的数据
      items = items.filter(item => !item.pageType || item.pageType === type);
      
      // 查找匹配formatId的数据
      const matchedItem = items.find((item: PageItem) => item.formatId === formatId);
      
      if (matchedItem) {
        console.log(`成功找到匹配的${type}数据:`, matchedItem);
        return {
          code: 200,
          msg: '获取成功',
          data: matchedItem
        };
      }
      
      // 如果没有找到匹配的formatId，返回该类型的第一条数据
      if (items.length > 0) {
        console.log(`未找到formatId为${formatId}的${type}数据，返回该类型的第一条数据:`, items[0]);
        return {
          code: 200,
          msg: '获取成功',
          data: items[0]
        };
      }
    }
    
    // 如果上面的方法失败，尝试其他备用方法
    console.log(`尝试使用备用API路径获取详情`);
    const backupUrl = `/api/hasfj/hasfjpages/list`;
    const backupParams = { pageType: type, pageNum: 1, pageSize: 10 };
    
    const backupResponse = await apiClient.get(backupUrl, { params: backupParams });
    
    if (backupResponse.status === 200 && backupResponse.data) {
      let items: PageItem[] = [];
      
      // 提取数据项
      if (Array.isArray(backupResponse.data)) {
        items = backupResponse.data;
      } else if (backupResponse.data.rows && Array.isArray(backupResponse.data.rows)) {
        items = backupResponse.data.rows;
      } else if (backupResponse.data.data && Array.isArray(backupResponse.data.data)) {
        items = backupResponse.data.data;
      }
      
      // 过滤掉不匹配类型的数据
      items = items.filter(item => !item.pageType || item.pageType === type);
      
      if (items.length > 0) {
        console.log(`备用API成功，返回该类型的第一条数据:`, items[0]);
        return {
          code: 200,
          msg: '获取成功',
          data: items[0]
        };
      }
    }
  } catch (e) {
    console.error('所有获取详情方法都失败:', e);
  }
  
  return {
    code: 500,
    msg: `获取${type}详情失败，请稍后重试`,
    data: null
  };
}

// 获取列表页面数据
export const getPageList = async (type: string, pageNum: number = 1, pageSize: number = 6) => {
  try {
    console.log(`开始请求${type}列表，页码:${pageNum}，每页数量:${pageSize}`);
    
    // 使用明确的类型参数，并修正API路径
    const url = `/hasfj/hasfjpages/list`;
    const params = { 
      pageType: type,
      pageNum, 
      pageSize
    };
    
    console.log(`请求URL: ${url}，参数:`, params);
    const response = await apiClient.get(url, { params });
    console.log(`API响应原始数据:`, response);
    
    if (response.status === 200) {
      console.log(`获取${type}列表成功，数据类型:`, typeof response.data, Array.isArray(response.data));
      
      // 检查响应数据结构
      if (!response.data) {
        console.error(`响应数据为空`);
        return { code: 500, msg: '响应数据为空', rows: [], total: 0 };
      }
      
      // 检查数据是否包含正确的类型
      let resultData;
      
      // 标准化响应结构
      if (response.data.code !== undefined) {
        // 后端已经返回标准格式
        console.log(`使用标准格式数据:`, response.data);
        resultData = response.data;
      } else if (Array.isArray(response.data)) {
        // 如果直接返回数组，构造标准格式
        console.log(`使用数组格式数据:`, response.data);
        // 过滤数组，确保只包含请求类型的数据
        const filteredData = response.data.filter((item: PageItem) => !item.pageType || item.pageType === type);
        resultData = {
          code: 200,
          msg: '操作成功',
          rows: filteredData,
          total: filteredData.length
        };
      } else if (response.data.rows) {
        // 使用rows字段
        console.log(`使用rows字段数据:`, response.data.rows);
        // 过滤数组，确保只包含请求类型的数据
        const filteredRows = Array.isArray(response.data.rows) 
          ? response.data.rows.filter((item: PageItem) => !item.pageType || item.pageType === type)
          : response.data.rows;
          
        resultData = {
          code: 200,
          msg: '操作成功',
          rows: filteredRows,
          total: response.data.total || filteredRows.length
        };
      } else if (response.data.data) {
        // 使用data字段
        console.log(`使用data字段数据:`, response.data.data);
        // 过滤数组，确保只包含请求类型的数据
        const filteredData = Array.isArray(response.data.data)
          ? response.data.data.filter((item: PageItem) => !item.pageType || item.pageType === type)
          : response.data.data;
          
        resultData = {
          code: 200,
          msg: '操作成功',
          rows: filteredData,
          total: response.data.total || filteredData.length
        };
      } else {
        // 尝试使用响应数据本身
        console.log(`使用响应数据本身:`, response.data);
        resultData = {
          code: 200,
          msg: '操作成功',
          rows: [response.data],
          total: 1
        };
      }
      
      // 最后检查一次确保返回的数据都是请求的类型
      if (Array.isArray(resultData.rows)) {
        resultData.rows = resultData.rows.filter((item: PageItem) => !item.pageType || item.pageType === type);
        resultData.total = resultData.rows.length;
      }
      
      return resultData;
    } else {
      console.error(`获取${type}列表失败:`, response);
      return {
        code: response.status,
        msg: '获取数据失败',
        rows: [],
        total: 0
      };
    }
  } catch (error) {
    console.error(`获取${type}列表失败:`, error);
    // 尝试使用备用API路径和方式
    try {
      // 尝试备用方案1：使用POST请求
      console.log(`尝试使用POST请求`);
      const url = `/hasfj/hasfjpages/list`;
      const data = { pageType: type, pageNum, pageSize };
      const response = await apiClient.post(url, data);
      
      if (response.status === 200 && response.data) {
        console.log(`POST请求成功:`, response.data);
        
        let resultData;
        if (Array.isArray(response.data)) {
          // 过滤数组，确保只包含请求类型的数据
          const filteredData = response.data.filter((item: PageItem) => !item.pageType || item.pageType === type);
          resultData = {
            code: 200,
            msg: '操作成功',
            rows: filteredData,
            total: filteredData.length
          };
        } else if (response.data.rows) {
          // 过滤数组，确保只包含请求类型的数据
          const filteredRows = Array.isArray(response.data.rows)
            ? response.data.rows.filter((item: PageItem) => !item.pageType || item.pageType === type)
            : response.data.rows;
            
          resultData = {
            code: 200,
            msg: '操作成功',
            rows: filteredRows,
            total: response.data.total || filteredRows.length
          };
        } else if (response.data.data) {
          // 过滤数组，确保只包含请求类型的数据
          const filteredData = Array.isArray(response.data.data)
            ? response.data.data.filter((item: PageItem) => !item.pageType || item.pageType === type)
            : response.data.data;
            
          resultData = {
            code: 200,
            msg: '操作成功',
            rows: filteredData,
            total: response.data.total || filteredData.length
          };
        }
        
        if (resultData) {
          return resultData;
        }
      }
    } catch (postError) {
      console.error(`POST请求失败:`, postError);
      
      // 尝试备用方案2：使用不同的API路径
      try {
        console.log(`尝试使用备用API路径`);
        const url = `/hasfj/hasfjpages/listByType/${type}`;
        const params = { pageNum, pageSize };
        const response = await apiClient.get(url, { params });
        
        if (response.status === 200 && response.data) {
          console.log(`备用API成功:`, response.data);
          
          let resultData;
          if (Array.isArray(response.data)) {
            // 过滤数组，确保只包含请求类型的数据
            const filteredData = response.data.filter((item: PageItem) => !item.pageType || item.pageType === type);
            resultData = {
              code: 200,
              msg: '操作成功',
              rows: filteredData,
              total: filteredData.length
            };
          } else {
            const rows = response.data.rows || response.data.data || [];
            // 过滤数组，确保只包含请求类型的数据
            const filteredRows = Array.isArray(rows)
              ? rows.filter((item: PageItem) => !item.pageType || item.pageType === type)
              : rows;
              
            resultData = {
              code: 200,
              msg: '操作成功',
              rows: filteredRows,
              total: response.data.total || filteredRows.length
            };
          }
          
          if (resultData) {
            return resultData;
          }
        }
      } catch (backupError) {
        console.error(`备用API也失败:`, backupError);
      }
    }
    
    return {
      code: 500,
      msg: `获取${type}列表失败，请稍后重试`,
      rows: [],
      total: 0
    };
  }
}

// 搜索内容
export const searchContent = async (keyword: string, type?: string) => {
  try {
    console.log(`开始搜索内容，关键词:${keyword}，类型:${type || '全部'}`);
    
    const params: any = { keyword };
    if (type) {
      params.pageType = type;
    }
    
    // 使用正确的API路径
    const url = `/hasfj/hasfjpages/search`;
    console.log(`搜索URL: ${url}，参数:`, params);
    
    const response = await apiClient.get(url, { params });
    
    console.log('搜索结果:', response.data);
    
    // 标准化响应结构
    if (response.status === 200) {
      let resultData;
      
      if (response.data.code !== undefined) {
        // 后端已经返回标准格式
        resultData = response.data;
      } else if (Array.isArray(response.data)) {
        // 如果直接返回数组，构造标准格式
        // 如果指定了类型，过滤数据
        const filteredData = type 
          ? response.data.filter((item: PageItem) => !item.pageType || item.pageType === type)
          : response.data;
          
        resultData = {
          code: 200,
          msg: '搜索成功',
          rows: filteredData,
          total: filteredData.length
        };
      } else if (response.data.rows) {
        // 使用rows字段
        // 如果指定了类型，过滤数据
        const filteredRows = type && Array.isArray(response.data.rows)
          ? response.data.rows.filter((item: PageItem) => !item.pageType || item.pageType === type)
          : response.data.rows;
          
        resultData = {
          code: 200,
          msg: '搜索成功',
          rows: filteredRows,
          total: response.data.total || filteredRows.length
        };
      } else if (response.data.data) {
        // 使用data字段
        // 如果指定了类型，过滤数据
        const filteredData = type && Array.isArray(response.data.data)
          ? response.data.data.filter((item: PageItem) => !item.pageType || item.pageType === type)
          : response.data.data;
          
        resultData = {
          code: 200,
          msg: '搜索成功',
          rows: filteredData,
          total: response.data.total || filteredData.length
        };
      } else {
        // 尝试使用响应数据本身
        resultData = {
          code: 200,
          msg: '搜索成功',
          rows: [response.data],
          total: 1
        };
      }
      
      return resultData;
    } else {
      console.error('搜索失败:', response);
      return {
        code: response.status,
        msg: '搜索失败',
        rows: [],
        total: 0
      };
    }
  } catch (error) {
    console.error('搜索内容失败:', error);
    
    // 尝试备用搜索方法
    try {
      console.log('尝试备用搜索方法');
      const backupUrl = `/api/hasfj/hasfjpages/search`;
      const backupParams: any = { keyword };
      if (type) {
        backupParams.pageType = type;
      }
      
      const backupResponse = await apiClient.get(backupUrl, { params: backupParams });
      
      if (backupResponse.status === 200 && backupResponse.data) {
        console.log('备用搜索成功:', backupResponse.data);
        
        let resultData;
        if (Array.isArray(backupResponse.data)) {
          // 如果指定了类型，过滤数据
          const filteredData = type 
            ? backupResponse.data.filter((item: PageItem) => !item.pageType || item.pageType === type)
            : backupResponse.data;
            
          resultData = {
            code: 200,
            msg: '搜索成功',
            rows: filteredData,
            total: filteredData.length
          };
        } else if (backupResponse.data.rows || backupResponse.data.data) {
          const rows = backupResponse.data.rows || backupResponse.data.data || [];
          // 如果指定了类型，过滤数据
          const filteredRows = type && Array.isArray(rows)
            ? rows.filter((item: PageItem) => !item.pageType || item.pageType === type)
            : rows;
            
          resultData = {
            code: 200,
            msg: '搜索成功',
            rows: filteredRows,
            total: backupResponse.data.total || filteredRows.length
          };
        }
        
        if (resultData) {
          return resultData;
        }
      }
    } catch (backupError) {
      console.error('备用搜索方法失败:', backupError);
    }
    
    return {
      code: 500,
      msg: '搜索失败，请稍后重试',
      rows: [],
      total: 0
    };
  }
}

// 获取二维码数据
export const getQRCodes = async (type: string, pageNum: number = 1, pageSize: number = 10) => {
  try {
    // 使用前端API专用的二维码接口
    const response = await apiClient.get('/hasfj/hasfjpages/qrcodes', {
      params: { type, pageNum, pageSize }
    });
    
    if (response.status === 200) {
      console.log('获取二维码数据成功:', response.data);
      // 标准化响应结构
      if (response.data.code !== undefined) {
        // 后端已经返回标准格式
        return response.data;
      } else if (Array.isArray(response.data)) {
        // 如果直接返回数组，构造标准格式
        return {
          code: 200,
          msg: '操作成功',
          rows: response.data,
          total: response.data.length
        };
      } else {
        // 其他情况，尝试使用data.data或构造空结果
        const rows = response.data.data || response.data.rows || [];
        const total = response.data.total || rows.length;
        return {
          code: 200,
          msg: '操作成功',
          rows,
          total
        };
      }
    } else {
      console.error('获取二维码数据失败:', response);
      return {
        code: response.status,
        msg: '获取二维码数据失败',
        rows: [],
        total: 0
      };
    }
  } catch (error) {
    console.error('获取二维码数据失败:', error);
    return {
      code: 500,
      msg: '获取二维码数据失败，请稍后重试',
      rows: [],
      total: 0
    };
  }
}

// 获取基础URL
export const getBaseUrl = (): string => {
  const protocol = window.location.protocol;
  const host = window.location.host;
  return `${protocol}//${host}`;
}

// 更新页面浏览量
export const updateViewCount = async (type: string, formatId: string) => {
  try {
    console.log(`开始更新${type}浏览量，formatId:`, formatId);
    
    // 使用页面详情API，后端会自动增加浏览量
    const url = `/hasfj/hasfjpages/updateViewCount`;
    const params = { 
      pageType: type,
      formatId: formatId
    };
    
    // 使用备用方案，如果updateViewCount API不存在，则使用页面详情API
    let response;
    try {
      response = await apiClient.get(url, { params });
      console.log(`更新浏览量响应:`, response);
    } catch (e) {
      console.warn(`更新浏览量API可能不存在，尝试使用页面详情API:`, e);
      // 尝试使用页面详情API，后端通常会在获取详情时自动更新浏览量
      response = await apiClient.get(`/hasfj/hasfjpages/get`, { 
        params: { 
          pageType: type, 
          formatId: formatId 
        } 
      });
    }
    
    if (response.status === 200) {
      return {
        code: 200,
        msg: '更新成功'
      };
    } else {
      console.error(`更新浏览量失败:`, response);
      return {
        code: response.status,
        msg: '更新失败'
      };
    }
  } catch (error) {
    console.error(`更新浏览量失败:`, error);
    return {
      code: 500,
      msg: '更新浏览量失败'
    };
  }
} 