import axios, { AxiosRequestConfig, AxiosRequestHeaders, AxiosResponse, InternalAxiosRequestConfig } from 'axios'
import { ElNotification, ElMessageBox, ElMessage, ElLoading } from 'element-plus'
import { getToken } from '@/utils/auth'
import errorCode from '@/utils/errorCode'
import { tansParams, blobValidate } from '@/utils/boyue'
import cache from '@/plugins/cache'
import { saveAs } from 'file-saver'
import useUserStore from '@/store/modules/user'
import router from '@/router'

let downloadLoadingInstance: any;
// 是否显示重新登录
export let isRelogin = { show: false };
//@ts-ignore
axios.defaults.headers['Content-Type'] = 'application/json;charset=utf-8'
import { GeekRequestConfig, GeekResponse } from '@/types/request'
// 创建axios实例
const service = axios.create({
  // axios中请求配置有baseURL选项，表示请求URL公共部分
  baseURL: import.meta.env.VITE_APP_BASE_API || '/prod-api',
  // 超时
  timeout: 10000
})

// 添加调试信息
console.log('API基础URL:', import.meta.env.VITE_APP_BASE_API || '/prod-api')
console.log('当前环境:', import.meta.env.MODE)

// request拦截器
service.interceptors.request.use((config) => {
  // 添加调试信息
  console.log('请求URL:', config.url)
  console.log('请求方法:', config.method)
  console.log('请求头:', config.headers)

  // 是否需要设置 token
  const isToken = (config.headers || {}).isToken === false
  // 是否需要防止数据重复提交
  const isRepeatSubmit = (config.headers || {}).repeatSubmit === false
  if (getToken() && !isToken) {
    // 让每个请求携带自定义token 请根据实际情况自行修改
    (<AxiosRequestHeaders>config.headers)['Authorization'] = 'Bearer ' + getToken()
    console.log('添加Token到请求头:', getToken())
  } else {
    console.log('未添加Token到请求头')
  }
  // get请求映射params参数
  if (config.method === 'get' && config.params) {
    let url = config.url + '?' + tansParams(config.params);
    url = url.slice(0, -1);
    config.params = {};
    config.url = url;
  }
  if (!isRepeatSubmit && (config.method === 'post' || config.method === 'put')) {
    const requestObj = {
      url: config.url,
      data: typeof config.data === 'object' ? JSON.stringify(config.data) : config.data,
      time: new Date().getTime()
    }
    const sessionObj = cache.session.getJSON('sessionObj')
    if (sessionObj === undefined || sessionObj === null || sessionObj === '') {
      cache.session.setJSON('sessionObj', requestObj)
    } else {
      const s_url = sessionObj.url;                // 请求地址
      const s_data = sessionObj.data;              // 请求数据
      const s_time = sessionObj.time;              // 请求时间
      const interval = 1000;                       // 间隔时间(ms)，小于此时间视为重复提交
      if (s_data === requestObj.data && requestObj.time - s_time < interval && s_url === requestObj.url) {
        const message = '数据正在处理，请勿重复提交';
        console.warn(`[${s_url}]: ` + message)
        return Promise.reject(new Error(message))
      } else {
        cache.session.setJSON('sessionObj', requestObj)
      }
    }
  }
  return config
}, error => {
  Promise.reject(error)
})

type ResponseType = {
  code: string | number,
  msg: string,
  data?: any,
  rows?: Array<any>,
  total?: any,
}

// 响应拦截器
// (value: V) => V | Promise<V>) | null) | null, options?: AxiosInterceptorOptions
service.interceptors.response.use(<T>(res: AxiosResponse<GeekResponse<T>, any>) => {
  // 添加调试信息
  console.log('响应状态码:', res.status)
  console.log('响应数据:', res.data)

  // 未设置状态码则默认成功状态
  const code = String(res.data.code || 200);
  console.log('业务状态码:', code)

  //获取错误信息
  const msg = errorCode[code] || res.data.msg || errorCode['default']
  console.log('响应消息:', msg)

  // 二进制数据则直接返回
  if (res.request.responseType === 'blob' || res.request.responseType === 'arraybuffer') {
    return res
  }

  // 检查是否是登录或验证码请求
  const isLoginRequest = res.config.url?.includes('/login');
  const isCaptchaRequest = res.config.url?.includes('/captchaImage');

  if (code === '401') {
    console.log('检测到401错误，需要重新登录')

    // 如果是登录或验证码请求，直接返回结果，不弹出登录框
    if (isLoginRequest || isCaptchaRequest) {
      console.log('登录或验证码请求，返回401错误')
      return Promise.resolve(res)
    }

    if (!isRelogin.show) {
      isRelogin.show = true;
      ElMessageBox.confirm('登录状态已过期，您可以继续留在该页面，或者重新登录', '系统提示', { confirmButtonText: '重新登录', cancelButtonText: '取消', type: 'warning' }).then(() => {
        isRelogin.show = false;
        useUserStore().logOut().then(() => {
          location.href = router.resolve('/login').href; // 修改为/login
        })
      }).catch(() => {
        isRelogin.show = false;
      });
    }
    return Promise.reject(new Error('无效的会话，或者会话已过期，请重新登录。'))
  } else if (code === '500') {
    ElMessage({ message: msg, type: 'error' })
    return Promise.reject(new Error(msg))
  } else if (code === '601') {
    ElMessage({ message: msg, type: 'warning' })
    return Promise.reject(new Error(msg))
  } else if (code !== '200') {
    // 如果是登录或验证码请求，不显示错误通知
    if (!isLoginRequest && !isCaptchaRequest) {
      ElNotification.error({ title: msg })
    }
    return Promise.reject('error')
  } else {
    return Promise.resolve(res)
  }
},
  error => {
    console.log('请求发生错误:', error)
    let { message } = error;
    if (message == "Network Error") {
      message = "后端接口连接异常";
    } else if (message.includes("timeout")) {
      message = "系统接口请求超时";
    } else if (message.includes("Request failed with status code")) {
      message = "系统接口" + message.substr(message.length - 3) + "异常";
    }
    console.log('错误消息:', message)

    // 检查是否是登录或验证码请求，如果是则不显示错误消息
    const isLoginRequest = error.config?.url?.includes('/login');
    const isCaptchaRequest = error.config?.url?.includes('/captchaImage');

    if (!isLoginRequest && !isCaptchaRequest) {
      ElMessage({ message: message, type: 'error', duration: 5 * 1000 })
    }
    return Promise.reject(error)
  }
)

// 通用下载方法
export async function download(url: string, params: any, filename: string, config: any) {
  downloadLoadingInstance = ElLoading.service({ text: "正在下载数据，请稍候", background: "rgba(0, 0, 0, 0.7)", })
  try {
    const res = await service.post(url, params, {
      transformRequest: [(params_1) => { return tansParams(params_1) }],
      headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
      responseType: 'blob',
      ...config
    })
    const data: Blob = res.data
    const isLogin = await blobValidate(data)
    if (isLogin) {
      const blob = new Blob([data])
      saveAs(blob, filename)
    } else {
      const resText = await data.text()
      const rspObj: ResponseType = JSON.parse(resText)
      const errMsg = errorCode[rspObj.code] || rspObj.msg || errorCode['default']
      ElMessage.error(errMsg)
    }
    downloadLoadingInstance.close()
  } catch (r) {
    console.error(r)
    ElMessage.error('下载文件出现错误，请联系管理员！')
    downloadLoadingInstance.close()
  }
}
const request = <T>(config: GeekRequestConfig) => service<GeekRequestConfig, AxiosResponse<GeekResponse<T>>>(config).then(res => res.data)
export function postAction(url: string, data?: any, isToken: boolean = true) {
  return request({ data, url, method: 'POST', headers: { isToken }, })
}
export function getAction(url: string, params?: any, isToken: boolean = true) {
  return request({ params, url, method: 'GET', headers: { isToken }, })
}
export function putAction(url: string, data?: any, isToken: boolean = true) {
  return request({ data, url, method: 'PUT', headers: { isToken }, })
}
export function deleteAction(url: string, data?: any, isToken: boolean = true) {
  return request({ data, url, method: 'DELETE', headers: { isToken }, })
}

export default request
